==> libgii-0.8.2.src.tar.bz2: libgii-0.8.2/gii/init.c <==
/* $Id: init.c,v 1.3 2002/02/22 21:51:05 soyt Exp $
******************************************************************************

   LibGII initialization.

   Copyright (C) 1997 Jason McMullan	[jmcc@ggi-project.org]
   Copyright (C) 1998 Marcus Sundberg	[marcus@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#include "config.h"
#include <ggi/internal/gii.h>
#include <ggi/internal/gii_debug.h>

#include "plat.h"


/* Exported variables */
EXPORTVAR uint32      _giiDebugState	= 0;
EXPORTVAR int         _giiDebugSync	= 0;
EXPORTVAR int         _gii_threadsafe	= 0;
EXPORTVAR void       *_gii_global_lock  = NULL;

/* Global variables */
void                 *_giiconfhandle	= NULL;
void                 *_gii_safe_lock	= NULL;

/* Static variables */
static int 	      _giiLibIsUp =0 ;
static char           giiconfstub[512] = GIICONFDIR;
static char          *giiconfdir = giiconfstub + GIITAGLEN;


/* 
 * Returns the directory where global config files are kept
 */

const char *giiGetConfDir(void)
{
#ifdef __WIN32__
	/* On Win32 we allow overriding of the compiled in path. */
	const char *envdir = getenv("GGI_CONFDIR");
	if (envdir) return envdir;
#endif
	return giiconfdir;
}


/*
 * Turn on thread safe operation
 */

int giiMTInit(void)
{
	_gii_threadsafe = 1;
	
	return 0;
}

/*
 * Initialize the structures for the library
 */

int giiInit(void)
{
	int err;
	const char *str;
	char *conffile;
	
	if (_giiLibIsUp>0) {
		/* Initialize only at first call. */
		_giiLibIsUp++;
		return 0;
	}

	err = ggInit();
	if (err != GGI_OK) {
		fprintf(stderr, "LibGII: unable to initialize LibGG\n");
		return err;
	}

	err = GGI_ENOMEM;
	if ((_gii_safe_lock = ggLockCreate()) == NULL) {
		fprintf(stderr,"LibGII: unable to initialize safe mutex.\n");
		goto out_ggexit;
	}

	if ((_gii_global_lock = ggLockCreate()) == NULL) {
		fprintf(stderr,"LibGII: unable to initialize global mutex.\n");
		goto out_destroy_safe;
	}

	conffile = malloc(strlen(giiGetConfDir()) + 1
			  + strlen(GIICONFFILE) +1);
	if (conffile == NULL) {
		fprintf(stderr,"LibGII: unable to allocate memory for config filename.\n");
		goto out_destroy_global;
	}
	sprintf(conffile, "%s%c%s",
		giiGetConfDir(), CHAR_DIRDELIM,	GIICONFFILE);
	err = ggLoadConfig(conffile, &_giiconfhandle);
	if (err != GGI_OK) {
		fprintf(stderr, "LibGII: fatal error - could not load %s\n",
			conffile);
		free(conffile);
		goto out_destroy_global;
	}
	free(conffile);

	str = getenv("GGI_DEBUG");
	if (str != NULL) {
		_giiDebugState = atoi(str);
		GIIDPRINT_CORE("Debugging=%d\n", _giiDebugState);
	}

	str = getenv("GGI_DEBUGSYNC");
	if (str != NULL) {
		_giiDebugSync = 1;
	}

	_giiLibIsUp++;

	return 0;

  out_destroy_global:
	ggLockDestroy(_gii_global_lock);
  out_destroy_safe:
	ggLockDestroy(_gii_safe_lock);
  out_ggexit:
	ggExit();

	return err;
}

int giiExit(void)
{
	GIIDPRINT_CORE("giiExit() called\n");
	if (!_giiLibIsUp)
		return -1;

	if (_giiLibIsUp > 1) {
		_giiLibIsUp--;
		return _giiLibIsUp;
	}

	GIIDPRINT_CORE("giiExit: really destroying.\n");

	ggFreeConfig(_giiconfhandle);
	ggLockDestroy(_gii_global_lock);
	ggLockDestroy(_gii_safe_lock);
	ggExit();
	_giiLibIsUp=0;

	GIIDPRINT_CORE("giiExit: done!\n");
	return 0;
}

==> libgii-0.9.0.src.tar.bz2: libgii-0.9.0/gii/init.c <==
/* $Id$
******************************************************************************

   LibGII initialization.

   Copyright (C) 1997 Jason McMullan	[jmcc@ggi-project.org]
   Copyright (C) 1998 Marcus Sundberg	[marcus@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include "config.h"
#include <ggi/internal/gii.h>
#include <ggi/internal/gii_debug.h>

#include "plat.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>



/* Exported variables */
uint32                _giiDebugState   = 0;
int                   _giiDebugSync    = 0;
int                   _gii_threadsafe  = 0;
void                 *_gii_global_lock = NULL;

/* Global variables */
void                 *_giiconfhandle	= NULL;
void                 *_gii_safe_lock	= NULL;
void                 *_gii_event_lock	= NULL;

/* Static variables */
static int 	      _giiLibIsUp =0 ;
static char           giiconfstub[512] = GIICONFDIR;
static char          *giiconfdir = giiconfstub + GIITAGLEN;


/* 
 * Returns the directory where global config files are kept
 */

const char *giiGetConfDir(void)
{
#if defined(__WIN32__) && !defined(__CYGWIN__)
	/* On Win32 we allow overriding of the compiled in path. */
	const char *envdir = getenv("GGI_CONFDIR");
	if (envdir) return envdir;
#endif
	return giiconfdir;
}


/*
 * Turn on thread safe operation
 */

int giiMTInit(void)
{
	_gii_threadsafe = 1;
	
	return 0;
}

/*
 * Initialize the structures for the library
 */

int giiInit(void)
{
	int err;
	const char *str;
	char *conffile;
	
	if (_giiLibIsUp>0) {
		/* Initialize only at first call. */
		_giiLibIsUp++;
		return 0;
	}

	err = ggInit();
	if (err != GGI_OK) {
		fprintf(stderr, "LibGII: unable to initialize LibGG\n");
		return err;
	}

	err = GGI_ENOMEM;
	if ((_gii_event_lock = ggLockCreate()) == NULL) {
		fprintf(stderr,"LibGII: unable to initialize event mutex.\n");
		goto out_ggexit;
	}

	if ((_gii_safe_lock = ggLockCreate()) == NULL) {
		fprintf(stderr,"LibGII: unable to initialize safe mutex.\n");
		goto out_destroy_event;
	}

	if ((_gii_global_lock = ggLockCreate()) == NULL) {
		fprintf(stderr,"LibGII: unable to initialize global mutex.\n");
		goto out_destroy_safe;
	}

	conffile = malloc(strlen(giiGetConfDir()) + 1
			  + strlen(GIICONFFILE) +1);
	if (conffile == NULL) {
		fprintf(stderr,"LibGII: unable to allocate memory for config filename.\n");
		goto out_destroy_global;
	}
#ifdef HAVE_SNPRINTF
	snprintf(conffile, strlen(giiGetConfDir()) + strlen(GIICONFFILE) + 2,
		"%s%c%s", giiGetConfDir(), CHAR_DIRDELIM, GIICONFFILE);
#else
	sprintf(conffile, "%s%c%s",
		giiGetConfDir(), CHAR_DIRDELIM,	GIICONFFILE);
#endif
	err = ggLoadConfig(conffile, &_giiconfhandle);
	if (err != GGI_OK) {
		fprintf(stderr, "LibGII: fatal error - could not load %s\n",
			conffile);
		free(conffile);
		goto out_destroy_global;
	}
	free(conffile);

	str = getenv("GGI_DEBUGSYNC");
	if (str != NULL) {
		_giiDebugSync = 1;
	}

	str = getenv("GGI_DEBUG");
	if (str != NULL) {
		_giiDebugState = atoi(str);
		GIIDPRINT_CORE("%s Debugging=%d\n",
				_giiDebugSync ? "sync" : "async",
				_giiDebugState);
	}

	_giiLibIsUp++;

	return 0;

  out_destroy_global:
	ggLockDestroy(_gii_global_lock);
  out_destroy_safe:
	ggLockDestroy(_gii_safe_lock);
  out_destroy_event:
	ggLockDestroy(_gii_event_lock);
  out_ggexit:
	ggExit();

	return err;
}

int giiExit(void)
{
	GIIDPRINT_CORE("giiExit() called\n");
	if (!_giiLibIsUp)
		return GGI_ENOTALLOC;

	if (_giiLibIsUp > 1) {
		_giiLibIsUp--;
		return _giiLibIsUp;
	}

	GIIDPRINT_CORE("giiExit: really destroying.\n");

	ggFreeConfig(_giiconfhandle);
	ggLockDestroy(_gii_global_lock);
	ggLockDestroy(_gii_safe_lock);
	ggLockDestroy(_gii_event_lock);
	ggExit();
	_giiLibIsUp=0;

	GIIDPRINT_CORE("giiExit: done!\n");
	return 0;
}
